@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'bundler/setup'
 
 # Set up coverage analysis
@@ -33,4 +34,6 @@ require 'molinillo'
 RSpec.configure do |config|
   # Enable flags like --only-failures and --next-failure
   config.example_status_persistence_file_path = '.rspec_status'
+  config.filter_run :focus => true
+  config.run_all_when_everything_filtered = true
 end
